package org.jboss.soa.qa.test;

import org.jboss.arquillian.container.test.api.Deployment;
import org.jboss.arquillian.container.test.api.RunAsClient;
import org.jboss.arquillian.junit.Arquillian;
import org.jboss.shrinkwrap.api.Archive;
import org.jboss.shrinkwrap.api.ShrinkWrap;
import org.jboss.shrinkwrap.api.importer.ZipImporter;
import org.jboss.shrinkwrap.api.spec.WebArchive;

import org.junit.Assert;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;
import org.junit.runner.RunWith;

import javax.activation.DataHandler;
import javax.activation.URLDataSource;
import javax.xml.namespace.QName;
import javax.xml.soap.AttachmentPart;
import javax.xml.soap.MessageFactory;
import javax.xml.soap.SOAPBodyElement;
import javax.xml.soap.SOAPConnection;
import javax.xml.soap.SOAPConnectionFactory;
import javax.xml.soap.SOAPConstants;
import javax.xml.soap.SOAPMessage;

import java.io.File;
import java.net.URL;
import java.util.Enumeration;

@RunWith(Arquillian.class)
public class MultipartContentTypeTest {

	private static final String PROJECT_NAME = "reproducer-eap-wrong-multipart";
	//	http://localhost:8080/reproducer-eap-wrong-multipart/testServlet
	private static final String TEST_SERVLET_URL = "http://localhost:8080/" + PROJECT_NAME + "/testServlet";
	private static final String IN_IMG_RESOURCE = "test.png";
	private static final String IN_IMG_NAME = IN_IMG_RESOURCE;

	@Deployment(order = 1, managed = true, testable = true)
	public static Archive createDeployment() {
		return ShrinkWrap
				.create(ZipImporter.class, PROJECT_NAME + ".war")
				.importFrom(
						new File("target" + File.separator + PROJECT_NAME
								+ ".war")).as(WebArchive.class);
	}

	@BeforeClass
	public static void prepareClass() throws Exception {
		Thread.sleep(5000L);
	}

	@Before
	public void prepare() {
	}

	@Test
	public void serverTest() throws Exception {
		//it fails
		System.out.println("This test runs from server.");
		testSendMultipartSoapMessage();
	}

	@Test
	@RunAsClient
	public void clientTest() throws Exception {
		//its OK
		System.out.println("This test runs from client.");
		testSendMultipartSoapMessage();
	}

	private void testSendMultipartSoapMessage() throws Exception {
		final MessageFactory msgFactory = MessageFactory.newInstance(SOAPConstants.SOAP_1_2_PROTOCOL);
		final SOAPMessage msg = msgFactory.createMessage();
		final SOAPBodyElement bodyElement = msg.getSOAPBody().addBodyElement(
				new QName("urn:ledegen:soap-attachment:1.0", "echoImage"));
		bodyElement.addTextNode("cid:" + IN_IMG_NAME);

		final AttachmentPart ap = msg.createAttachmentPart();
		ap.setDataHandler(getResource(IN_IMG_RESOURCE));
		ap.setContentId(IN_IMG_NAME);
		msg.addAttachmentPart(ap);

		final SOAPConnectionFactory conFactory = SOAPConnectionFactory.newInstance();
		final SOAPConnection connection = conFactory.createConnection();
		final SOAPMessage response = connection.call(msg, new URL(TEST_SERVLET_URL));

		final String contentTypeWeHaveSent = getBodyElementTextValue(response);
		assertContentTypeStarts("multipart/related", contentTypeWeHaveSent);
	}

	private void assertContentTypeStarts(final String expectedStart, final String actual) {
		if (!actual.startsWith(expectedStart)) {
			Assert.fail("We have send request with Content-Type " + actual + ", but expected start is " + expectedStart);
		}
	}

	private String getBodyElementTextValue(final SOAPMessage msg) throws Exception {
		final SOAPBodyElement bodyElement = (SOAPBodyElement) msg.getSOAPBody().getChildElements().next();
		final String result = bodyElement.getTextContent();
		return result;
	}

	private DataHandler getResource(final String resource) throws Exception {
		final URL imageUrl = getResourceUrl(resource, this.getClass().getClassLoader());
		return new DataHandler(new URLDataSource(imageUrl));
	}

	public static URL getResourceUrl(String path, final ClassLoader cl) throws Exception {
		final Enumeration<URL> e = cl.getResources(path);
		if (e.hasMoreElements()) {
			return e.nextElement();
		}
		return null;
	}
}
